window.uiUserAddedClick = false;


app.saved = true;
window.onbeforeunload = function(evt) {
    if (!app.saved) {
        var message = "Document is not saved. You will lost the changes if you leave the page.";
        if (typeof evt == "undefined") {
            evt = window.event;
        }
        if (evt) {
            evt.returnValue = message;
        }
        return message;
    }
}

app.AppView = Backbone.View.extend({
    el: '#app',
    width: 0,

    afterZoom: function() {
        $(".paper-scroller").css("padding-left", "0px");
        $(".paper-scroller").css("padding-top", "0px");
        $("#paper").css("margin-bottom", "0px");
        $("#paper").css("margin-right", "0px");
        var cells = _.chain(this.graph.getCells())
            .filter(function(c) {
                return !c.IsLink })
            .map(function(c) {
                var b = c.getBBox();
                return { w: b.x + b.width, h: b.y + b.height }
            })
            .value();
        var w = _.chain(cells).pluck("w").max().value();
        var h = _.chain(cells).pluck("h").max().value();
    },

    initialize: function() {
        this.initializePaper();
        if (!isLive) {
            this.initializeSelection();
            this.initializeHalo();
            this.initializeInlineTextEditor();
        }

        var t = this;
        this.graph.on('change', function(cell) {
            if (cell && !cell.Link && "m" in cell.attributes) {
                var b = cell.getBBox();
                var w = b.x + b.width;
                if (w < t.minWidth) w = t.minWidth;
                if (w != t.width) {
                    t.width = w;
                    $("svg").attr("width", w);
                    $("#paper").css("width", w + "px");
                }
                var h = b.y + b.height;
                if (h < t.minHeight) h = t.minHeight;
                if (h != t.height && !isLive) {
                    t.height = h;
                    $("svg").attr("height", h);
                    $("#paper").css("height", h + "px");
                }
            }
        })
    },


    initializeInlineTextEditor: function() {
        var cellViewUnderEdit;

        var closeEditor = _.bind(function() {
            if (this.textEditor) {
                this.textEditor.remove();
                cellViewUnderEdit.options.interactive = true;
                this.textEditor = cellViewUnderEdit = undefined;

            }
        }, this);

        this.paper.on('cell:pointerdblclick', function(cellView, evt) {
            closeEditor();

            var vTarget = V(evt.target);
            var text;
            var cell = cellView.model;

            switch (cell.get('type')) {
                case 'qad.Question':
                case 'qad.Answer':
                    text = joint.ui.TextEditor.getTextElement(evt.target);
                    cellView.textEditPath = 'answer';
                    break;
            }

            if (text) {
                this.textEditor = new joint.ui.TextEditor({ text: text });
                this.textEditor.render(this.paper.el);
                this.textEditor.on('text:change', function(newText) {
                    var cell = cellViewUnderEdit.model;
                    cell.prop(cellViewUnderEdit.textEditPath, newText);
                    if (cellViewUnderEdit.optionId) {
                        this.textEditor.options.text = cellViewUnderEdit.$('.option.option-' + cellViewUnderEdit.optionId + ' .option-text')[0];
                    }
                }, this);
                cellViewUnderEdit = cellView;
                cellViewUnderEdit.options.interactive = false;
            }
        }, this);

        $(document.body).on('click', _.bind(function(evt) {
            var text = joint.ui.TextEditor.getTextElement(evt.target);
            if (this.textEditor && !text) {
                closeEditor();
            }
        }, this));
    },

    initializeHalo: function() {
        this.paper.on('cell:pointerup', function(cellView, evt) {
            if (cellView.model instanceof joint.dia.Link) return;

            var t = this;

            var halo = new joint.ui.Halo({
                graph: this.graph,
                paper: this.paper,
                cellView: cellView,
                useModelGeometry: true,
                type: 'toolbar',
                boxContent: false,
                handles: [{
                    name: 'remove',
                    events: { pointerdown: 'removeElement' }
                }, {
                    name: 'clone',
                    events: { /*pointerdown: 'startCloning', pointermove: 'doClone', pointerup: 'stopCloning' */
                    pointerup: function() { 
                        var cell  = cellView.model;
                        if (cell) { 
                            var type = cell.get("type");
                            if (type=="qad.Question") {
                                var model = cell.get("m");
                                var attrs = _.clone(model.attributes);
                                delete attrs.shape;
                                var a = JSON.parse(JSON.stringify(attrs));
                                _.each(a.input, function(i) {
                                    if (i.value && i.value.link) i.value.link=false;
                                    if (i.value && i.value.isEvent) {
                                        i.value.isEvent=false;
                                        i.value.value=false;
                                    }
                                })

                                var c = new app.FuncModel();
                                c.set(_.clone(a));

                                window.uiUserAddedClick = true;
                                c.updateShape();
                    
                                var b = cell.getBBox();
                                window.positionCellFreePlace(c.get("shape"), b.x, b.y, 10);

                                c.trigger("change:event");
                            } else 
                                t.graph.addCell(cell.clone());
                        }
                    }
                }
                }, {
                    name: 'unlink',
                    events: { pointerdown: 'unlinkElement' }
                },
                {
                    name: 'addaddress',
                    events: { pointerdown: 'addAddress' }
                },
                {
                    name: 'showhelp',
                    events: {
                        pointerdown: function(e) {
                            var position = {
                                my: "left-top",
                                at: "right-top",
                                of: halo.$el[0],
                                offsetX: 10,
                                offsetY: 0,
                                maxTop: 65
                              };
                            app.showHelpPanel(rightView.model.attributes, position, true);
                        }
                    }
                }]
            });

            halo.on('action:remove:pointerdown', function() {
                this.selection.reset();
            }, this)

            halo.render();

        }, this);
    },

    initializeSelection: function() {
        document.body.addEventListener('keydown', _.bind(function(evt) {

            var code = evt.which || evt.keyCode;
            // Do not remove the element with backspace if we're in inline text editing.
            if ((/*code === 8|| */ code === 46) && !this.textEditor && this.selection.first()) {

                this.selection.first().remove();
                this.selection.reset();
                return false;
            }

        }, this), false);

        var selection = this.selection = new app.Selection;
        var selectionView = new app.SelectionView({ model: selection, paper: this.paper });
        this.paper.on('cell:pointerup', function(cellView) {
            if (!cellView.model.isLink()) {
                selection.reset([cellView.model]);
            }
        });
        this.paper.on('blank:pointerdown', function() { selection.reset([]) });

        selection.on('add reset', this.onSelectionChange, this);
    },

    initializePaper: function() {

        this.graph = new joint.dia.Graph;

        var h = $(window).height();
        var w = $(window).width() / 6 * 5;
        if (isLive) w =  $(window).width();
        this.minWidth = w;
        this.width = w;
        this.minHeight = h;
        this.height = h;

        this.paper = new joint.dia.Paper({
            el: this.$('#paper'),
            model: this.graph,
            width: w - 40,
            height: h - 95,
            padding: 0,
            margin: 0,
            gridSize: 1,
            snapLinks: { radius: 10 },
            interactive: isLive ? false : true,
            validateConnection: function(cellViewS, magnetS, cellViewT, magnetT, end, linkView) {
                if (magnetS && magnetS.getAttribute('type') === 'input') return false; //исходящий event 
                if (magnetS && magnetS.getAttribute('type') === 'in') return false; //исходящиц - просто входящиц 
                if (magnetT && magnetT.getAttribute('type') === 'out') return false; //входящий - исходящий

                if (!magnetT) return cellViewT.isCanLinkTo ? true : false;

                //если ссылка в просто входящий порт
                if (magnetT.getAttribute('type') === 'in') {

                    if (cellViewS.model.get("type")=="qad.Question" && 
                        (cellViewT.model.get("type")=="qad.Address" || cellViewT.model.get("type")=="qad.Addressnotevent" || cellViewT.model.get("type")=="qad.Storage")) 
                        return true;

                    //порт уже занят 
                    if (magnetT.getAttribute('isFree')!="1") { 
                        var ts = getTargetSource(linkView.model);
                        if (ts && ts.trg && ts.trg.port && ts.trg.port.length>0 && ts.trg.port[0]===magnetT) ;
                        else return false;
                    }
                    
                    //не event 
                    if (cellViewS.model.get("type")=="qad.Question" || cellViewS.model.get("type")=="qad.Storage"  
                        || cellViewS.model.get("type")=="qad.Value" || cellViewS.model.get("type")=="qad.Addressnotevent" ) 
                        return true;

                    //event 
                    if (cellViewT.model.get("type") == "qad.Question" && cellViewT.model.get("eventValue") !== "timer" ) { 
                        if (cellViewS.model.get("type")=="qad.Address" ||  cellViewS.model.get("type")=="qad.Tag")
                            return true;
                    }

                    return false;
                }

                //если event из функции, то должен быть со специальным атрибутом (указан групповой адрес)
                if (magnetT.getAttribute('type') === 'input' && cellViewS.model.get("type")=="qad.Question") {
                    if (magnetS.getAttribute('enableEvent')!="1") 
                        return false;
                }

                //если ссылка в event port
                if (magnetT.getAttribute('type') === 'input') {
                    //в верхний порт можно только timer 
                    if (cellViewS.model.get("type")!=="qad.Timer")
                        return false;
                }

                //сам в себя 
                if (cellViewS === cellViewT) return false;

                return true;
            },

            validateMagnet: function(cellView, magnet) {
                if (magnet.getAttribute('mag') == 'passive') return false;

                //input - это входяший event 
                //in - это просто входящий 
                if (magnet.getAttribute('type') == 'input' || magnet.getAttribute('type') !== 'in') return true;

                return false;
            },

            defaultLink: new joint.dia.Link({
                router: { name: 'manhattan' },
                connector: { name: 'rounded' },
                attrs: {
                    '.marker-target': { d: 'M 10 0 L 0 5 L 10 10 z', fill: '#6a6c8a', stroke: '#6a6c8a' },
                    '.connection': {
                        stroke: '#6a6c8a',
                        'stroke-width': 2
                            //filter: { name: 'dropShadow', args: { dx: 1, dy: 1, blur: 2 } }
                    },
                }
            })
        });

        this.paperScroller = new joint.ui.PaperScroller({
            autoResizePaper: true,
            paper: this.paper
        });
        this.paper.on('blank:pointerdown', this.paperScroller.startPanning);

        this.paperScroller.$el.css({
            width: w - 20,
            height: h - 70,
            padding: 0,
            margin: 0,
        });

        $("#app").append(this.paperScroller.render().el);
        this.commandManager = new joint.dia.CommandManager({ graph: this.graph });

        //провели стрелки, диалоги, изменения 
        window.changeTarget = function(l, remove, userClick) { 
            var p = getTargetSource(l);
            var t = l._t;

            //если таргет до этого был до удаляить его 
            if (t) {
                var c = t.port[0];

                //ссылка в event 
                if (c.getAttribute("isEvent")=="1") {
                    if (isEventFree(t.cell, l)) t.cell.changeFree("event", true);
                    l.attr({
                        '.connection':{stroke: '#6a6c8a'},
                        '.marker-target':{stroke: 'orange', fill:'#6a6c8a'}
                    });

                //ссылка в простой входящий порт функции 
                }else if (c.getAttribute("type")=="in" && t.cell.get("type")=="qad.Question") { 
                    var id = t.port[0].getAttribute("option-id");

                    if (p.src && t.cell.get("type")=="qad.Question") {
                        if ((p.src.cell.get("type")=="qad.Question" || p.src.cell.get("type")=="qad.Storage" 
                            || p.src.cell.get("type")=="qad.Value" || p.src.cell.get("type")=="qad.Addressnotevent")) {
                            t.cell.changeFree(id, true);
                            removeLinkFunc2Func(t.cell.get("m"), id);   
                        }else if (p.src.cell.get("type")=="qad.Address" || p.src.cell.get("type")=="qad.Tag") { 
                            t.cell.changeFree(id, true);        
                            removeLinkFunc2Func(t.cell.get("m"), id);   
                        }
                    }
                }
            }

            if (p.trg && p.trg.cell && !remove) { 
                if (userClick) { 
                    var c = p.trg.port[0];
                    if (c.getAttribute("isEvent")=="1"){
                        p.trg.cell.changeFree("event", false);
                        l.attr({
                            '.connection':{stroke: 'orange'},
                            '.marker-target':{stroke: 'orange', fill:'orange'}
                        });

                    //ссылка в простой входящий порт функции 
                    }else if (c.getAttribute("type")=="in") { 
                        if (p.src && p.trg.cell.get("type")=="qad.Question"){
                            //func 2 func 
                            if  (p.src.cell.get("type")=="qad.Question") {
                                var inNum = p.trg.port[0].getAttribute("option-id");
                                p.trg.cell.changeFree(inNum, false, false);

                                var tS = p.src.cell.get("type"), inps = p.trg.cell.get("m").get("input");
                                if (_.isArray(inps) && inNum<inps.length) {     
                                    var inp = inps[inNum];
                                    var valAddrr;

                                    var isEvent = false;

                                    var _createLink = function() {
                                        var isSuccess = false;
                                        if (!isEvent) {
                                            if (isEnabledDataTypeForPort("storage", inp)) {
                                                createLink2Func(p.trg.cell.get("m"), inNum, p.src.cell);      
                                                isSuccess = true;
                                            }
                                        }else { 
                                            if (isEnabledDataTypeForPort("object", inp)) {
                                                if (valAddrr) {
                                                    createLink2Func(p.trg.cell.get("m"), inNum, p.src.cell, true, valAddrr.value); 
                                                    l.attr({
                                                        '.connection':{stroke: 'orange'},
                                                        '.marker-target':{stroke: 'orange', fill:'orange'}
                                                    });

                                                    p.trg.cell.changeFree(inNum, false, true);
                                                    isSuccess = true;
                                                }
                                            }
                                        }

                                        if (!isSuccess) { 
                                            new app.ModalError({
                                                error : "The field does not support this datatype",
                                                callback : function() {l.remove();} 
                                            })
                                        }
                                    } 

                                    if (p.trg.cell.get("m").get("event") !== "timer" && p.src.port[0].getAttribute("enableEvent")) { 
                                        var outNum = p.src.port[0].getAttribute("option-id");
                                        var outs = p.src.cell.get("m").get("output");
                                        if (outNum < outs.length  && _.isArray(outs[outNum].values)) {
                                            valAddrr = _.find(outs[outNum].values, function(m){ return m.type=="object" && _.isString(m.value) && m.value.length>0});
                                            if (valAddrr) {
                                                new app.ModalQuestion({
                                                    text:"Is it event link?", 
                                                    buttonOK:"Yes", 
                                                    buttonCancel:"No", 
                                                    class: "info",
                                                    ok : function() {isEvent = true; _createLink(); },
                                                    cancel : function() {_createLink() }
                                                })
                                            }else 
                                                _createLink();
                                        }    
                                    }else 
                                        _createLink();
                                }
                            //не event
                            }else if  (p.src.cell.get("type")=="qad.Storage" 
                                || p.src.cell.get("type")=="qad.Value" || p.src.cell.get("type")=="qad.Addressnotevent" ) { 

                                var inNum = p.trg.port[0].getAttribute("option-id");
                                p.trg.cell.changeFree(inNum, false, false);

                                var tS = p.src.cell.get("type"), inps = p.trg.cell.get("m").get("input");
                                if (_.isArray(inps) && inNum<inps.length) { 
                                    var inp = inps[inNum];

                                    //проверяем на совместимость типов данных 
                                    if (
                                        (tS=="qad.Addressnotevent" && isEnabledDataTypeForPort("object", inp)) ||
                                        (tS=="qad.Storage" && isEnabledDataTypeForPort("storage", inp)) ||
                                        (tS=="qad.Value" && isEnabledDataTypeForPort("value", inp))                                   
                                        ) {
                                        createLink2Func(p.trg.cell.get("m"), inNum, p.src.cell);       
                                    }else { 
                                        p.trg.cell.trigger("updateShape");
                                        new app.ModalError({
                                            error : "The field does not support this datatype",
                                            callback : function() {l.remove();} 
                                        })
                                    }

                                }

                            //event 
                            }else if (p.src.cell.get("type")=="qad.Address" || p.src.cell.get("type")=="qad.Tag" ) {
                                var inNum = p.trg.port[0].getAttribute("option-id");
                                p.trg.cell.changeFree(inNum, false, true);

                                var tS = p.src.cell.get("type"), inps = p.trg.cell.get("m").get("input");
                                if (_.isArray(inps) && inNum<inps.length) { 
                                    var inp = inps[inNum];

                                    //проверяем на совместимость типов данных 
                                    if (isEnabledDataTypeForPort("object", inp) || 
                                        (p.src.cell.get("type")=="qad.Tag" && isEnabledDataTypeForPort("tag", inp))
                                        ) {
                                        createLink2Func(p.trg.cell.get("m"), inNum, p.src.cell);       
                                    }else { 
                                        p.trg.cell.trigger("updateShape");
                                        new app.ModalError({
                                            error : "The field does not support this datatype",
                                            callback : function() {l.remove();} 
                                        })
                                    }
                                }

                            }
                        }else if (p.src && p.src.cell.get("type")=="qad.Question"  && (p.trg.cell.get("type")=="qad.Address" || p.trg.cell.get("type")=="qad.Storage" || p.trg.cell.get("type")=="qad.Addressnotevent")){
                                var outNum = p.src.port[0].getAttribute("option-id");

                                var tS = p.trg.cell.get("type"), outs = p.src.cell.get("m").get("output");
                                if (_.isArray(outs) && outNum<outs.length) { 
                                    var out = outs[outNum];
                                }

                        }
                    }
                }

                l._t = p.trg;
            }else { 
                l._t=false;
            }

            //удаляем ссылку между функциями -
            if (remove && p.trg && p.trg.cell && p.trg.port && p.trg.port.length>0) { 
                var t = p.trg.port[0].getAttribute("type");
                if (p.trg.cell.get("type")=="qad.Question" && t=="in") { 
                    var inNum = p.trg.port[0].getAttribute("option-id");
                    p.trg.cell.changeFree(inNum, true);
                    removeLinkFunc2Func(p.trg.cell.get("m"), inNum);   
                }
            }   
        }

        this.graph.on('add', function(cell) { 
            if (cell.isLink()) {
                var p = getTargetSource(cell);
                if (p.src && p.src.cell && p.src.port) { 
                    var t = p.src.cell.get("type");
                    if (t!="qad.Question" && t!="qad.Addressnotevent" && t!="qad.Value" && t!="qad.Storage") {
                        cell.attr({
                            '.connection':{stroke: 'orange'},
                            '.marker-target':{stroke: 'orange', fill:'orange'}
                        });
                    }
                }
            }
        })

        this.graph.on('remove', function(cell) { 
            cell.set("remove", true);
            if (cell.isLink()) changeTarget(cell, true)
            else { 
                rightView.changed = false;
            }
        })

        this.paper.on("cell:pointerup", function(v, e) {
            var cell = v.model;
            if (cell.isLink()) {
                var p = getTargetSource(v.model);
                if (!p.src || !p.trg) cell.remove();
                else if (!cell.get("remove")) changeTarget(cell, false, true); 
            }
        })

        this.graph.on("add", function(m) {
            app.saved = false;
            if (!m.isLink()) {
                if (window.uiUserAddedClick) {
                    window.positionCellFreePlace(m, 20, 20, 10);
                }else{
                    var b = m.getBBox();
                    var x = b.x, y=b.y;
                    window.positionCellFreePlace(m, x, y, 10);
                }

                var t = m.get("type"); 
                if (t=="qad.Question" || t=="qad.Tag" || t=="qad.Address" || t=="qad.Timer"  || t=="qad.Storage" || t=="qad.Value" || t=="qad.Addressnotevent") {
                    appView.selection.reset([m]);
                }

                window.uiUserAddedClick = false;
            }else if (m.isLink()) { 
                //console.log("adad");
                //changeTarget(m);      
            }

            cloningElement = false;
        });
    },

    onSelectionChange: function(collection) {
    },

    // Show a message in the statusbar.
    status: function(m) {
    },

    addAnswer: function() {
        var a = app.Factory.createAnswer(tr("leftmenu.elements.Comment"));
        window.uiUserAddedClick = true;
        this.graph.addCell(a);
        this.status(tr("app.commentadded"));
    },

    addAddress: function() {
        var a = app.Factory.createAddress();
        window.uiUserAddedClick = true;
        this.graph.addCell(a);
        this.status(tr("app.address_added"));
    },

    addTimer: function() {
        var a = app.Factory.createTimer();
        window.uiUserAddedClick = true;
        this.graph.addCell(a);
        this.status(tr("app.timer_added"));
    },

    addTag: function() {
        var a = app.Factory.createTag();
        window.uiUserAddedClick = true;
        this.graph.addCell(a);
        this.status(tr("app.tag_added"));
    },

    addStorage: function() {
        var a = app.Factory.createStorage();
        window.uiUserAddedClick = true;
        this.graph.addCell(a);
        this.status(tr("app.storage_added"));
    },

    addValue: function() {
        var a = app.Factory.createValue();
        window.uiUserAddedClick = true;
        this.graph.addCell(a);
        this.status(tr("app.value_added"));
    },

    addAddressNotEvent: function() {
        var a = app.Factory.createAddressNotEvent();
        window.uiUserAddedClick = true;
        this.graph.addCell(a);
        this.status(tr("app.address2_added"));
    },


    previewDialog: function() {
        var cell = this.selection.first();
        var el = qad.renderDialog(app.Factory.createDialogJSON(this.graph, cell));

        $('#preview').empty();
        $('#preview').append($('<div>', { 'class': 'background' }));
        $('#preview').append(el).show();

        $('#preview .background').on('click', function() {
            $('#preview').empty();
        });
    },

    clear: function() {
        this.graph.clear();
    },

});
