var app = app || {};
app._helpPanels = {};

app._helpPanel_getText = function (category, func, suffix) {
  return tr("functions." + category.name + ".json." + func.name + suffix);
}

app.showHelpPanel = function (func, position, permanent) {
  if (app._helpPanels[func.name] == null) {
    // Default panel position
    if (position == null) {
      position = {
        my: "right-top",
        at: "right-bottom",
        of: ".top-bar",
        offsetX: -20,
        offsetY: 20
      };
    }

    // Find category (Alert, DALI, ...)
    var category = null;
    for (let i = 0; i < functions.length; i++) {
      const categoryIter = functions[i];
      for (let j = 0; j < categoryIter.funcs.length; j++) {
        const categoryFunc = categoryIter.funcs[j];
        if (categoryFunc.name == func.name) {
          category = categoryIter;
          break;
        }
      }
      if (category != null)
        break;
    }

    // Table with inputs
    var inputsTable = "";
    if (func.input.length > 0) {
      inputsTable += "<table><tr>";
      for (let i = 1; i <= 4; i++) {
        inputsTable += "<th>" + tr("help.inputs.header." + i) + "</th>";
      }
      inputsTable += "</tr>";
      
      for (let i = 0; i < func.input.length; i++) {
        const input = func.input[i];
        var suffix = ".input." + (i + 1);
        inputsTable += "<tr>";
        inputsTable += "<td>" + app._helpPanel_getText(category, func, suffix) + "</td>";
        inputsTable += "<td>" + app._helpPanel_getText(category, func, suffix + ".init") + "</td>";
        inputsTable += "<td>[" + input.t.join(", ") + "]</td>";
        inputsTable += "<td>" + app._helpPanel_getText(category, func, suffix + ".description") + "</td>";
        inputsTable += "</tr>";
      }
      inputsTable += "</table>";
    }

    // Table with outputs
    var outputsTable = "";
    if (func.output.length > 0) {
      outputsTable += "<table><tr>";
      for (let i = 1; i <= 4; i++) {
        outputsTable += "<th>" + tr("help.outputs.header." + i) + "</th>";
      }
      outputsTable += "</tr>";
      
      for (let i = 0; i < func.output.length; i++) {
        const output = func.output[i];
        var suffix = ".output." + output.n;
        outputsTable += "<tr>";
        outputsTable += "<td>" + app._helpPanel_getText(category, func, suffix) + "</td>";
        outputsTable += "<td>" + app._helpPanel_getText(category, func, suffix + ".init") + "</td>";
        outputsTable += "<td>[" + output.t.join(", ") + "]</td>";
        outputsTable += "<td>" + app._helpPanel_getText(category, func, suffix + ".description") + "</td>";
        outputsTable += "</tr>";
      }
      outputsTable += "</table>";
    }
    
    // Create panel
    var panel = jsPanel.create({
      theme: {
        bgPanel: "#e6e6e6",
        bgContent: "#e6e6e6",
        colorHeader: "#000",
        colorContent: "#000"
      },
      position: position,
      boxShadow: 1,
      headerTitle: app._helpPanel_getText(category, func, ".title"),
      headerControls: {
        size: "xs"
      },
      content: "<p><b>" + app._helpPanel_getText(category, func, ".title") + "</b></p>" +
        "<p><b>" + tr("help.category") + "</b> " + tr("functionanames." + category.name) + "</p>" +
        "<p><b>" + tr("help.description") + "</b></p>" +
        "<p>" + app._helpPanel_getText(category, func, ".comment") + "</p>" +
        inputsTable + outputsTable,
      contentSize: {
        width: function () { return 600; },
        height: "auto"
      },
      maximizedMargin: [60, 10, 10, 10],
      dragit: {
        containment: 2
      },
      onstatuschange: function(){
        $(this).toggleClass("jsPanel-transparent", this.status == "normalized");
      },
      callback: function () {
        $(this).addClass("jsPanel-transparent");
      },
      onclosed: function () {
        app._helpPanels[func.name] = null;
      }
    });
    app._helpPanels[func.name] = {
      panel: panel,
      permanent: permanent === true
    };
  }
  else {
    var panelInfo = app._helpPanels[func.name];
    if (panelInfo.panel.status != "minimized") {
      panelInfo.panel.front();
    }
    if (permanent === true && !panelInfo.permanent) {
      panelInfo.permanent = true;
    }
  }
}

app.highlightHelpPanel = function (func) {
  var panelInfo = app._helpPanels[func.name];
  if (panelInfo && panelInfo.panel.status != "minimized") {
    panelInfo.panel.front();
  }
}

app.hideHelpPanel = function (func, closePermanent) {
  var panelInfo = app._helpPanels[func.name];
  if (panelInfo && (!panelInfo.permanent || closePermanent)) {
    panelInfo.panel.close();
    app._helpPanels[func.name] = null;
  }
}