var spacer = '\xA0\xA0\xA0';

joint.shapes.qad = {};

joint.shapes.qad.Answer = joint.shapes.basic.Generic.extend({
    markup: '<g class="rotatable"><g class="scalable"><rect/></g><text/></g>',

    defaults: joint.util.deepSupplement({
        type: 'qad.Answer',
        size: { width: 100, height: 40 },
        attrs: {
            'rect': { fill: '#4b4a67', stroke: 'none', width: 100, height: 40 /*, rx: 3, ry: 3 */ },
            'text': { text: '', lineHeight: 15, 'ref-x': .5, 'ref-y': .5, ref: 'rect', 'y-alignment': 'middle', 'x-alignment': 'middle', fill: '#f6f6f6', 'font-family': 'Arial, helvetica, sans-serif' }
        },
        inPorts: [{ id: 'in', label: 'In' }],
        outPorts: [{ id: 'yes', label: 'Yes' }, { id: 'no', label: 'No' }]

    }, joint.shapes.basic.Generic.prototype.defaults),

    initialize: function() {
        joint.shapes.basic.Generic.prototype.initialize.apply(this, arguments);
        this.attr('text/text', this.get('answer'));
        this.on('change:answer', function() {
            this.attr('text/text', this.get('answer') || spacer);
        }, this);
    }
});

joint.util.measureText = function(text, attrs) {
    var fontSize = parseInt(attrs.fontSize, 10) || 10;
    var svgDocument = V('svg').node;
    var textElement = V('<text><tspan></tspan></text>').node;
    var textSpan = textElement.firstChild;
    var textNode = document.createTextNode('');

    textSpan.appendChild(textNode);
    svgDocument.appendChild(textElement);
    document.body.appendChild(svgDocument);

    var lines = text.split('\n');
    var width = 0;

    // Find the longest line width.
    _.each(lines, function(line) {
        textNode.data = line;
        var lineWidth = textSpan.getComputedTextLength();
        width = Math.max(width, lineWidth);
    });

    var height = lines.length * 15 + 25;

    V(svgDocument).remove();

    return { width: width, height: height };
};

joint.shapes.qad.AnswerView = joint.dia.ElementView.extend({
    isCanLinkTo: true,

    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        this.autoresize();
        this.listenTo(this.model, 'change:answer', this.autoresize, this);
    },

    autoresize: function() {
        var dim = joint.util.measureText(this.model.get('answer') || '\xA0\xA0\xA0\xA0\xA0', {
            fontSize: this.model.attr('text/font-size')
        });
        console.log('resize', dim);
        this.model.resize(dim.width, dim.height);
    }
});

joint.shapes.qad.Question = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g class="scalable"><rect class="body"/></g><text class="question-text"/><g class="options"></g><g class="inPorts"/></g>',
    portMarkup: '<g class="port eventport"><circle class="port-body port-in-body"/><text class="port-label port-in-text"/></g>',

    outOptionMarkup: '<g class="option outoption"><rect class="option-rect"/><path class="btn-remove-option" d="M0,0 10,0 10,3 0,3z"/><text class="option-text"/><g class="option-port"><circle class="port-body"/><text class="port-label"/></g></g>',

    inOptionMarkup: '<g class="option inoption"><rect class="option-rect"/><text class="option-text"/><g class="option-port"><circle class="port-body"/><text class="port-label"/></g></g>',
    inOptionMarkupWithoutPort: '<g class="option inoption"><rect class="option-rect"/><text class="option-text"/></g>',

    defaults: joint.util.deepSupplement({
        type: 'qad.Question',
        size: { width: 1, height: 1 },
        eventValue: "timer",
        tttemplate: null,
        optionHeight: 30,
        questionHeight: 45,
        paddingBottom: 20,
        minWidth: 150,

        inPorts: [{ id: "in" }],
        outPorts: [],

        attrs: {
            free: true,
            '.': { magnet: false },
            '.body': {
                width: 150,
                height: 250,
                stroke: 'none',
                fill: {
                    type: 'linearGradient',
                    stops: [
                        { offset: '0%', color: '#FEB663' },
                        { offset: '100%', color: '#31D0C6' }
                    ],
                    attrs: { x1: '0%', y1: '0%', x2: '0%', y2: '100%' }
                }
            },
            text: { 'font-family': 'Arial' },
            '.question-text': { 'font-size': 13, dy: 5, fill: 'white', 'ref-x': .5, 'ref-y': 17, 'x-alignment': 'middle', ref: '.body', style: { 'text-shadow': '1px 1px 0px gray' } },

            //порты
            '.port-body': { magnet: true },
            '.port-label': { 'pointer-events': 'none'},

            //опции и текст
            '.options': { ref: '.body', 'ref-x': 0 },
            '.option .option-rect': { stroke: 'white', 'stroke-width': 1, 'stroke-opacity': .5, 'fill-opacity': .5, fill: 'white', ref: '.body', 'ref-width': 1},
            '.option-text': { 'font-size': 12, dy: 5, fill: '#4b4a67', 'y-alignment': .4, 'x-alignment': 20, "font-family": "FontAwesome" },
            '.inoption .option-text': { fill: '#fff', "ref-x": 20, "ref-y": 10 },
            '.inoption .option-rect': { fill: 'black' },
            '.btn-remove-option': {'x-alignment': 5, 'y-alignment': 9, cursor: 'pointer' },

            //порты у опций
            '.option-port': { ref: '.body', 'y-alignment': 15 },
            '.option .port-body': { stroke: 'none', r: 14, fill: '#31d0c6' },
            '.outoption .option-port': { 'y-alignment': 15, 'ref-dx': 0 },
            '.outoption .port-body': { type: 'out' },
            '.inoption .port-body': {fill: '#AAA', type: 'in', isFree: "1" },
            '.option-port .port-label': {fill: '#000', "font-size": 10, dx: 16 },
            '.inoption .option-port .port-label': { "text-anchor": "end", dx: -16 },

            //верхний порт
            '.inPorts': { 'ref-x': 0.5, "ref-y": 0, mag: 'passive' },
            '.inPorts .port-body': { fill: 'red', r: 14, type: 'input', isFree: "1", isEvent: "1" },
            '.inPorts .port-label': { 'font-size': 10, fill: 'black', "ref-x": 62, "ref-y": -23 },
        }
    }, joint.shapes.basic.Generic.prototype.defaults),


    changeFree: function(portNumber, value, isEvent) {
        if (portNumber == "event") {
            this.set("free", value);
            //if (_.isUndefined(free)) free = true;
            if (value) {
                this.attr({ ".port-in-body": { fill: "red", isFree: "1" } });
            } else {
                this.attr({ ".port-in-body": { fill: "orange", isFree: "0" } });
            }
        } else {
            var a = {};

            if (value) {
                a[".option-in-" + portNumber + " .port-body"] = { isFree: "1", fill: '#AAA' };
            } else {
                if (!isEvent)
                    a[".option-in-" + portNumber + " .port-body"] = { isFree: "0", fill: "#31d0c6" };
                else
                    a[".option-in-" + portNumber + " .port-body"] = { isFree: "0", fill: "orange" };
            }

            this.attr(a);
        }
    },

    init: function() {
        var t = this;

        //изменилось значение у модели event
        this.get("m").on("change:event", function() {
            var m = t.get("m");
            var e = m.get("event");

            if (e !== "timer") e = "event";
            t.set("eventValue", e);

            _.each(appView.graph.getLinks(), function(l) {
                var p = getTargetSource(l);
                if (p.trg && p.trg.cell && p.trg.cell == t) {
                    if (p.src && p.src.cell) {
                        var src = p.src.cell;
                        if (e == "timer" && (src.get("type") == "qad.Address" || src.get("type") == "qad.Tag")) l.remove();
                        else if (e != "timer" && src.get("type") == "qad.Timer") l.remove();
                    } else l.remove();
                }
            });

        })
    },

    initialize: function() {
        this.on('change:inOptions', this.onChangeOptions, this);

        if (this.get("m")) this.init();

        var t = this;

        this.on('change:outOptions', function() {
            this.onChangeOptions();
        });

        this.on('change:question', function() {
            this.attr('.question-text/text', this.get('question') || '');
            this.autoresize();
        }, this);

        this.on('change:questionHeight', function() {
            this.attr('.options/ref-y', this.get('questionHeight'), { silent: true });
            this.autoresize();
        }, this);

        this.on('change:optionHeight', this.autoresize, this);

        this.attr('.options/ref-y', this.get('questionHeight'), { silent: true });
        this.attr('.question-text/text', this.get('question'), { silent: true });

        this.onChangeOptions();
        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    onChangeOptions: function() {
        var outoptions = this.get('outOptions');
        var inoptions = this.get('inOptions');
        var size = this.get('size');
        var optionHeight = this.get('optionHeight');

        var attrs = this.get('attrs');
        _.each(attrs, function(attrs, selector) {
            if (attrs.dynamic) {
                this.removeAttr(selector, { silent: true });
            }
        }, this);

        var offsetY = 0;
        var attrsUpdate = {};

        var width = 0;

        _.each(inoptions, function(option) {
            var v = option.value;
            var text = option.label;

            if (_.isObject(v)) {
                text = getIcon(v.type) + " " + text;
                if (_.isString(v.value)) {
                    text += (text.length > 0 ? " = " : "") + v.value;
                }
            }

            var w = joint.util.measureText(text, { fontSize: 12 }).width / 1.15;
            if (width < w) width = w;

            var selector = '.option-in-' + option.id;
            attrsUpdate[selector] = { transform: 'translate(0, ' + offsetY + ')', dynamic: true };
            attrsUpdate[selector + ' .option-rect'] = { height: optionHeight, dynamic: true };
            attrsUpdate[selector + ' .option-port .port-body'] = { port: "in-" + option.id, dynamic: true };
            attrsUpdate[selector + ' .option-text'] = { text: text || option.text, dynamic: true };

            offsetY += optionHeight;
        }, this);

        _.each(outoptions, function(option) {
            var selector = '.option-out-' + option.id;
            attrsUpdate[selector] = { transform: 'translate(0, ' + offsetY + ')', dynamic: true };
            attrsUpdate[selector + ' .option-rect'] = { height: optionHeight, dynamic: true };

            var attrPort = { port: option.id, dynamic: true, enableEvent: option.enableEvent ? "1" : "0" };
            if (option.enableEvent) attrPort["fill"] = "orange";
            attrsUpdate[selector + ' .option-port .port-body'] = attrPort;

            attrsUpdate[selector + ' .option-text'] = { text: option.text, dynamic: true };

            var w = joint.util.measureText(option.text, { fontSize: 12 }).width / 1.15;
            if (width < w) width = w;

            //if (option.portLabel) {
            attrsUpdate[selector + ' .option-port .port-label'] = { text: option.portLabel || "" };
            //}
            offsetY += optionHeight;
        }, this);

        this.attr(attrsUpdate);
        this.autoresize(width);
    },

    autoresize: function(woptions) {
        if (!woptions) woptions = 0;

        var options = this.get('inOptions') || [];
        var inoptions = this.get('outOptions') || [];
        var gap = this.get('paddingBottom') || 20;
        var height = (options.length + inoptions.length) * this.get('optionHeight') + this.get('questionHeight') + gap;
        var width = joint.util.measureText(this.get('question'), {
            fontSize: this.attr('.question-text/font-size')
        }).width;

        this.resize(Math.max(this.get('minWidth') || 150, width, woptions), height);

    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};

        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';

        attrs[portCircleSelector] = { port: { id: port.id, type: type /*, label: pl */ } };

        if (selector === '.outPorts') { attrs[portSelector]['ref-dy'] = 0; }

        return attrs;
    },

    addOption: function(option) {
        var options = JSON.parse(JSON.stringify(this.get('options')));
        options.push(option);
        this.set('options', options);
    },

    removeOption: function(id) {
        var options = JSON.parse(JSON.stringify(this.get('options')));
        this.set('options', _.without(options, _.findWhere(options, { id: id })));
    },

    changeOption: function(id, option) {
        if (!option.id) {
            option.id = id;
        }
        var options = JSON.parse(JSON.stringify(this.get('options')));
        options[_.findIndex(options, { id: id })] = option;
        this.set('options', options);
    }
}));

joint.shapes.qad.QuestionView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    events: {
        // 'click .btn-add-option': 'onAddOption',
        'click .btn-remove-option': 'onRemoveOption'
    },

    //если изменился тип скрипта (event или timer)
    onChangeEventType: function() {
        var eventValue = this.model.get("eventValue");

       // console.log(eventValue);
        if (eventValue != "timer") {
            this.$("g.eventport").remove();
            //this.update();
        } else {
            var p = this.elTopPortMarkup.clone();
            this.$(".inPorts").append(p.node);
            this.model.attr({ ".port-in-text": { text: getIcon("timer"), dx: -50, "font-size": 12, "font-family": "FontAwesome" } });
            this.update();
        }
    },

    initialize: function() {
        //app.FuncModel.updateShape
        this.listenTo(this.model, "updateShape", this.onChangeEventType, this);
        this.listenTo(this.model, "change", this.onChangeEventType, this);

        this.listenTo(this.model, "change:eventValue", this.onChangeEventType, this);

        this.listenTo(this.model, 'change:inOptions', this.renderOptions, this);

        this.listenTo(this.model, 'change:outOptions', function() {
            this.renderOptions();

            var l = _.each(appView.graph.getLinks(), function(l) {
                var p = getTargetSource(l);
                if (p.src.port) {
                    var id = $(p.src.port).attr("id")
                    if (id) {
                        if ($("#" + id).length == 0) l.remove();
                    } else l.remove();
                } else l.remove();
            });

        });

        this.elTopPortMarkup = V(this.model.portMarkup);
        this.onChangeEventType();

        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    },

    renderMarkup: function() {
        joint.dia.ElementView.prototype.renderMarkup.apply(this, arguments);
        this.$options = this.$('.options');
        this.elInOption = V(this.model.inOptionMarkup);
        this.elOutOption = V(this.model.outOptionMarkup);
        this.elInOptionWithoutPort = V(this.model.inOptionMarkupWithoutPort);
        this.renderOptions();
    },

    renderOptions: function() {
        this.$options.empty();
        var t = this;

        var inputs;
        if (this.model.get("m")) {
            inputs = this.model.get("m").get("input");
        } else {
            inputs = this.model.get("m_attr").input;
        }

        function addOption(option, isInput, i) {
            var className = 'option-' + (isInput ? "in" : "out") + "-" + option.id;

            var elOption;

            if (!isInput) {
                elOption = t.elOutOption.clone().addClass(className);
            } else {
                if (isInput && _.isArray(inputs) && inputs.length > i) {
                    var v = inputs[i].value;

                    if (v.value === false || v.link) {
                        elOption = t.elInOption.clone().addClass(className);
                    }
                }

                if (_.isUndefined(elOption))
                    elOption = t.elInOptionWithoutPort.clone().addClass(className);
            }

            $(".port-body", $(elOption.node)).attr('option-id', option.id);
            elOption.attr('option-id', option.id);
            if (isInput && option.event) elOption.addClass('event');
            if (isInput && option.error) elOption.addClass('error');

            t.$options.append(elOption.node);
        }

        _.each(this.model.get('inOptions'), function(option, i) { addOption(option, true, i) }, this);
        _.each(this.model.get('outOptions'), function(option) { addOption(option, false) }, this);

        this.update();
        if (this.model.get("eventValue") != "timer")
            setTimeout(this.onChangeEventType(), 1000);


        //this.onChangeEventType();
    },


    onRemoveOption: function(evt) {
        var n = V(evt.target.parentNode);
        if (n.hasClass("outoption")) {
            var id = n.attr("option-id"),
                m = this.model.get("m");
            m.hideOutOption(id);
        }
    }
}));

joint.shapes.qad.Address = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/><g class="inPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/><text class="port-label"/></g>',

    defaults: joint.util.deepSupplement({
        value: false,
        type: 'qad.Address',
        size: { width: 120, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#ABDAC6', stroke: 'none', width: 120, height: 40, /*rx: 3, ry: 3 */ },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61874), 'ref-x': 20, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.event_elements.Address"), 'ref-x': 37, "ref-y": ".35" },

            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': { fill: "orange", r: 14, type: 'out' },

            '.inPorts': { "ref-y": .5, "ref-x": .01, mag: 'passive' },
            '.inPorts .port-body': { fill: "#AAA", r: 14, type: 'in', mag: 'passive' },

        },
        outPorts: [{ id: 'out', label: 'Yes' }],
        inPorts: [{ id: 'in', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Address";
            this.attr('.label/text', a);
        }, this);

        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};
        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';
        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    },
}));

joint.shapes.qad.AddressView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));


joint.shapes.qad.AddAddress = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/></g>',

    defaults: joint.util.deepSupplement({
        value: 60,
        type: 'qad.AddAddress',
        size: { width: 115, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#DDCFE6', stroke: 'none', width: 115, height: 40 /*, rx: 3, ry: 3 */ },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61463), 'ref-x': 10, "ref-y": ".35" },
            '.label': { text: "60", 'ref-x': 27, "ref-y": ".35" },
            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': {fill: "orange", r: 14, type: 'out', isTimer: "1" },
        },
        outPorts: [{ id: 'out', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        joint.shapes.basic.Generic.prototype.initialize.apply(this, arguments);

        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = 0;
            this.attr('.label/text', a);
            var w = a.toString().length * 8.5 + 30;
            if (w < 115) w = 115;
            this.resize(w, 40);
            this.attr("rect/width", w);
        }, this);

        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};

        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';

        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };

        return attrs;
    },
}));
joint.shapes.qad.Timer = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/></g>',

    defaults: joint.util.deepSupplement({
        value: 60,
        type: 'qad.Timer',
        size: { width: 115, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#DDCFE6', stroke: 'none', width: 115, height: 40 /*, rx: 3, ry: 3 */ },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61463), 'ref-x': 10, "ref-y": ".35" },
            '.label': { text: "60", 'ref-x': 27, "ref-y": ".35" },
            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': {fill: "orange", r: 14, type: 'out', isTimer: "1" },
        },
        outPorts: [{ id: 'out', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        joint.shapes.basic.Generic.prototype.initialize.apply(this, arguments);

        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = 0;
            this.attr('.label/text', a);
            var w = a.toString().length * 8.5 + 30;
            if (w < 115) w = 115;
            this.resize(w, 40);
            this.attr("rect/width", w);
        }, this);

        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};

        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';

        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };

        return attrs;
    },
}));

joint.shapes.qad.TimerView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));

joint.shapes.qad.Tag = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/></g>',

    defaults: joint.util.deepSupplement({
        value: false,
        type: 'qad.Tag',
        size: { width: 115, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#ABB7DD', stroke: 'none', width: 115, height: 40 /*, rx: 3, ry: 3 */ },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61484), 'ref-x': 10, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.event_elements.Tag"), 'ref-x': 27, "ref-y": ".35" },
            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': { fill: "orange", r: 14, type: 'out' },
        },
        outPorts: [{ id: 'out', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        joint.shapes.basic.Generic.prototype.initialize.apply(this, arguments);

        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Tag";
            this.attr('.label/text', a);
            var w = a.length * 8.5 + 30;
            if (w < 115) w = 115;
            this.resize(w, 40);
            this.attr("rect/width", w);
        }, this);

        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {

        var attrs = {};

        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';

        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    },
}));

joint.shapes.qad.TagView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));

joint.shapes.qad.Addressnotevent = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/><g class="inPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/><text class="port-label"/></g>',

    defaults: joint.util.deepSupplement({
        value: false,
        type: 'qad.Addressnotevent',
        size: { width: 120, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#ABDAC6', stroke: 'none', width: 120, height: 40 },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61874), 'ref-x': 20, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.event.Address"), 'ref-x': 37, "ref-y": ".35" },

            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': { fill: "#31d0c6", r: 14, type: 'out', notevent: false },

            '.inPorts': { "ref-y": .5, "ref-x": .01, mag: 'passive' },
            '.inPorts .port-body': { fill: "#AAA", r: 14, type: 'in', mag: 'passive' },

        },
        outPorts: [{ id: 'out', label: 'Yes' }],
        inPorts: [{ id: 'in', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Address";
            this.attr('.label/text', a);
        }, this);
        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};
        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';
        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    }
}));

joint.shapes.qad.AddressnoteventView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));

joint.shapes.qad.Storage = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/><g class="inPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/><text class="port-label"/></g>',

    defaults: joint.util.deepSupplement({
        value: false,
        type: 'qad.Storage',
        size: { width: 120, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#99CCCC', stroke: 'none', width: 120, height: 40 },
            'text': { fill: "black", },

            '.title': { text: String.fromCharCode(61888), 'ref-x': 20, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.event.Storage"), 'ref-x': 37, "ref-y": ".35" },

            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': { fill: "#31d0c6", r: 14, type: 'out', notevent: false },

            '.inPorts': { "ref-y": .5, "ref-x": .01, mag: 'passive' },
            '.inPorts .port-body': { fill: "#AAA", r: 14, type: 'in', mag: 'passive' },
        },
        outPorts: [{ id: 'out', label: 'Yes' }],
        inPorts: [{ id: 'in', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Storage";
            this.attr('.label/text', a);
            var w = Math.max(114, a.length * 8 + 45);
            this.resize(w, 40);
            this.attr("rect/width", w);
        }, this);
        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};
        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';
        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    }
}));

joint.shapes.qad.StorageView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));

joint.shapes.qad.Value = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/><text class="port-label"/></g>',

    defaults: joint.util.deepSupplement({
        value: false,
        type: 'qad.Value',
        size: { width: 115, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#CCFF66', stroke: 'none', width: 115, height: 40 },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61508), 'ref-x': 10, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.elements.Value"), 'ref-x': 27, "ref-y": ".35" },
            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': { fill: "#31d0c6", r: 14, type: 'out', notevent: false },
        },
        outPorts: [{ id: 'out', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Value";
            this.attr('.label/text', a);
            var w = Math.max(114, a.length * 8 + 35);
            this.resize(w, 40);
            this.attr("rect/width", w);
        }, this);
        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};
        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';
        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    }
}));

joint.shapes.qad.ValueView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));
