joint.shapes.qad = {};

joint.shapes.qad.Answer = joint.shapes.basic.Generic.extend({
    markup: '<g class="rotatable"><g class="scalable"><rect/></g><text/></g>',

    defaults: joint.util.deepSupplement({
        type: 'qad.Answer',
        size: { width: 100, height: 40 },
        attrs: {
            'rect': { fill: '#4b4a67', stroke: 'none', width: 100, height: 40 /*, rx: 3, ry: 3 */ },
            'text': { text: '', lineHeight: 15, 'ref-x': .5, 'ref-y': .5, ref: 'rect', 'y-alignment': 'middle', 'x-alignment': 'middle', fill: '#f6f6f6', 'font-family': 'Arial, helvetica, sans-serif' }
        },
        inPorts: [{ id: 'in', label: 'In' }],
        outPorts: [{ id: 'yes', label: 'Yes' }, { id: 'no', label: 'No' }]

    }, joint.shapes.basic.Generic.prototype.defaults),

    initialize: function() {
        joint.shapes.basic.Generic.prototype.initialize.apply(this, arguments);
        this.attr('text/text', this.get('answer'));
        this.on('change:answer', function() {
            this.attr('text/text', this.get('answer'));
        }, this);
    }
});

joint.util.measureText = function(text, attrs) {
    var fontSize = parseInt(attrs.fontSize, 10) || 10;
    var svgDocument = V('svg').node;
    var textElement = V('<text><tspan></tspan></text>').node;
    var textSpan = textElement.firstChild;
    var textNode = document.createTextNode('');

    textSpan.appendChild(textNode);
    svgDocument.appendChild(textElement);
    document.body.appendChild(svgDocument);

    var lines = text.split('\n');
    var width = 0;

    // Find the longest line width.
    _.each(lines, function(line) {
        textNode.data = line;
        var lineWidth = textSpan.getComputedTextLength();
        width = Math.max(width, lineWidth);
    });

    var height = lines.length * 15 + 25;

    V(svgDocument).remove();

    return { width: width, height: height };
};

joint.shapes.qad.AnswerView = joint.dia.ElementView.extend({
    isCanLinkTo: true,

    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        this.autoresize();
        this.listenTo(this.model, 'change:answer', this.autoresize, this);
    },

    autoresize: function() {
        var dim = joint.util.measureText(this.model.get('answer'), {
            fontSize: this.model.attr('text/font-size')
        });
        this.model.resize(dim.width, dim.height);
    }
});

joint.shapes.qad.Question = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g class="scalable"><rect class="body"/></g><text class="question-text"/><g class="options"></g><g class="inPorts"/></g>',
    portMarkup: '<g class="port eventport"><circle class="port-body port-in-body"/><text class="port-label port-in-text"/></g>',
    outOptionMarkup: '<g class="option outoption"><rect class="option-rect"/><text class="option-text"/><g class="option-rows"></g><g class="option-port"><circle class="port-body"/><text class="port-label"/></g></g>',

    inOptionMarkup: '<g class="option inoption m2"><rect class="option-rect"/><text class="option-text"/><text class="option-value"/><g class="option-port"><circle class="port-body"/><text class="port-label"/></g></g>',
    inOptionMarkupWithoutPort: '<g class="option inoption m1"><rect class="option-rect"/><text class="option-text"/><text class="option-value"/></g>',

    inOptionMarkupAddr: '<g class="option inoption a"><rect class="option-rect"/><text class="option-text"/><text class="option-value"/><text class="icon-value"/><g class="option-port"><circle class="port-body"/><text class="port-label"/></g></g>',
    inOptionMarkupAddrWithoutPort: '<g class="option inoption a1"><rect class="option-rect"/><text class="option-text"/><text class="option-value"/><text class="icon-value"/></g>',

    optionRow: '<g class="option-row row"><rect class="option-row-rect"/><text class="option-row-text"/><text class="option-row-value"/></g>',
    optionRowAddr: '<g class="option-row row"><rect class="option-row-rect"/><text class="option-row-text"/><text class="option-row-value"/><text class="icon-value"/></g>',

    tttemplate: null,

    defaults: joint.util.deepSupplement({
        type: 'qad.Question',
        size: { width: 1, height: 1 },

        optionHeight: 30,
        optionRowHeight: 17,
        questionHeight: 45,
        paddingBottom: 20,
        minWidth: 150,
        maxWidth: 350,

        inPorts: [{ id: "in" }],
        outPorts: [],

        attrs: {
            free: true,
            '.': { magnet: false },
            '.body': {
                width: 150,
                height: 250,
                stroke: 'none',
                fill: {
                    type: 'linearGradient',
                    stops: [
                        { offset: '0%', color: '#FEB663' },
                        { offset: '100%', color: '#31D0C6' }
                    ],
                    attrs: { x1: '0%', y1: '0%', x2: '0%', y2: '100%' }
                }
            },
            text: { 'font-family': 'Arial' },
            '.question-text': { 'font-size': 13, dy: 5, fill: 'white', 'ref-x': .5, 'ref-y': 17, 'x-alignment': 'middle', ref: '.body', style: { 'text-shadow': '1px 1px 0px gray' } },

            //порты общее 
            '.port-body': { magnet: true },
            '.port-label': { 'pointer-events': 'none' },

            //опции и текст
            '.options': { ref: '.body', 'ref-x': 0 },
            '.option .option-rect': { stroke: 'white', 'stroke-width': 1, 'stroke-opacity': .5, 'fill-opacity': .5, fill: 'white', ref: '.body', 'ref-width': 1},
            '.option-text': { 'font-size': 11, dy: 5, fill: '#4b4a67', 'y-alignment': .4, 'x-alignment': 7, "font-family": "FontAwesome", "font-size": 14 },
            '.inoption .option-text': { fill: '#fff', "ref-x": 22, "ref-y": 10  },
            '.inoption .option-rect': { fill: 'black' },
            //только для live                         
            '.option-row-text': { fill: '#4b4a67', 'y-alignment': .4, 'x-alignment': 7, "font-family": "FontAwesome", "font-size": 13 },
            '.option-row-value': { fill: '#4b4a67', 'y-alignment': .4, "font-family": "FontAwesome", "font-size": 13, "font-style": "italic" },
            '.option-value': { "ref-x": 13, fill: '#4b4a67', 'y-alignment': .7, "font-family": "FontAwesome", "font-size": 13, "font-style": "italic" },
            '.icon-value': { text: String.fromCharCode(61545), fill: '#4b4a67', 'y-alignment': .6, "font-family": "FontAwesome", cursor: "pointer", 'font-size': 14, "ref-x": 110 },
            '.inoption .icon-value': { fill: "white"},
            '.inoption .option-value': { fill: '#fff' },

            //порты у опций
            '.option-port': { ref: '.body', /* 'ref-dx': 0,*/ 'y-alignment': 15 },
            '.option .port-body': { stroke: 'none', r: 14, fill: '#31d0c6' },
            '.outoption .option-port': { 'y-alignment': 15, 'ref-dx': 0 }, //?
            '.outoption .port-body' : { type: 'out' },
            '.inoption .port-body': { fill: '#AAA', type: 'in', isFree: "1"},
            '.option-port .port-label': { fill: '#000', "font-size": 10, dx: 16 },
            '.inoption .option-port .port-label': { "text-anchor": "end", dx: -16 }, //?
            
            //верхний порт
            '.inPorts': { 'ref-x': 0.5, "ref-y": 0, mag: 'passive' },
            '.inPorts .port-body': { fill: 'red', r: 14, type: 'input', isFree: "1" },
            '.inPorts .port-label': { 'font-size': 10, fill: 'black', "ref-x": 62, "ref-y": -23 },
        }
    }, joint.shapes.basic.Generic.prototype.defaults),

    changeFree: function() {
        var free = this.get("free");
        if (_.isUndefined(free)) free = true;
        if (free) {
            this.attr({ ".port-in-body": { fill: "red", isFree: "1" } });
        } else {

            this.attr({ ".port-in-body": { fill: "orange", isFree: "0" } });
        }
    },

    init: function() {
        var t = this;
        this.get("m").on("change:event", function() {
            var m = t.get("m");
            var e = m.get("event");

            var l = _.find(appView.graph.getLinks(), function(l) {
                var p = getTargetSource(l);
                return p.trg && p.trg.cell && p.trg.cell == t;
            });
            if (l) {
                var p = getTargetSource(l);
                if (p.src && p.src.cell) {
                    var src = p.src.cell;
                    if (e == "timer" && src.get("type") != "qad.Timer") l.remove();
                    else if (e != "timer" && src.get("type") == "qad.Timer") l.remove();
                } else l.remove();
            }
        })
    },

    initialize: function() {
        this.on('change:inOptions', this.onChangeOptions, this);

        if (this.get("m")) this.init();

        var t = this;

        this.on('change:outOptions', function() {
            this.onChangeOptions();
        });

        this.on('change:labelEvent', function() {
            var pl = this.get("labelEvent");
            /*if (pl=="timer") { 
                this.attr({".port-in-text" : { text: String.fromCharCode(61463), "font-family" : "FontAwesome", "font-size":14, dx:-27 }});
                this.attr({".port-in-body" : {isTimer : "1"}});
            }else {*/
            dx = pl.length * 3.7 + 27;
            this.attr({ ".port-in-text": { text: pl, dx: -dx, "font-size": 12, "font-family": "FontAwesome" } });
            this.attr({ ".port-in-body": { isTimer: "0" } });
            //this.trigger('updateShape');
            //}
        });

        this.on('change:free', function() {
            this.changeFree();
            //this.trigger('updateShape');
        })

        this.on('change:question', function() {
            this.attr('.question-text/text', this.get('question') || '');
            this.autoresize();
        }, this);

        this.on('change:questionHeight', function() {
            this.attr('.options/ref-y', this.get('questionHeight'), { silent: true });
            this.autoresize();
        }, this);

        this.on('change:optionHeight', this.autoresize, this);

        this.attr('.options/ref-y', this.get('questionHeight'), { silent: true });
        this.attr('.question-text/text', this.get('question'), { silent: true });

        this.onChangeOptions();
        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);

        //addToMapping 
        var t = this;
        var options = this.get('inOptions');
        if (_.isArray(options)) {
            _.each(options, function(o) {
                if (_.isObject(o.value)) {
                    var v = o.value.value;
                    if (_.isString(v)) {
                        if (o.value.type == "object") {
                            if (!mapping.o[v]) mapping.o[v] = [];
                            mapping.o[v].push({ model: t, options: { t: "in", id: o.id } });
                        } else if (o.value.type == "storage") {
                            if (!mapping.s[v]) mapping.s[v] = [];
                            mapping.s[v].push({ model: t, options: { t: "in", id: o.id } });
                        }
                    }
                }
            })
        }
        options = this.get('outOptions');
        if (_.isArray(options)) {
            _.each(options, function(o) {
                if (_.isArray(o.values)) {
                    _.each(o.values, function(value, i) {
                        if (_.isObject(value)) {
                            var v = value.value;
                            if (_.isString(v)) {
                                if (value.type == "object") {
                                    if (!mapping.o[v]) mapping.o[v] = [];
                                    mapping.o[v].push({ model: t, options: { t: "out", id: o.id, v: i } });
                                } else if (value.type == "storage") {
                                    if (!mapping.s[v]) mapping.s[v] = [];
                                    mapping.s[v].push({ model: t, options: { t: "out", id: o.id, v: i } });
                                }
                            }
                        }
                    })
                }
            })
        }
    },

    changeValue: function(o, v) {
        var selector;
        if (o.t == "in") {
            var options = this.get('inOptions');
            var p = _.find(options, function(op) {
                return op.id == o.id });
            if (!_.isUndefined(p.value.v)) selector = '.option-in-' + o.id + " .option-value";
            p.value.v = v;
            //this.trigger('updateShape');
            this.set("inOptions", options);
            //this.trigger('updateShape');
            this.onChangeOptions();
            //this.trigger('updateShape');
        } else if (o.t == "out") {
            var options = this.get('outOptions');
            var p = _.find(options, function(op) {
                return op.id == o.id });
            if (!_.isUndefined(p.values[o.v].v)) selector = '.option-out-' + o.id + "-row-" + o.v + ' .option-row-value';
            p.values[o.v].v = v;
            //this.trigger('updateShape');
            this.set("outOptions", options);
            //this.trigger('updateShape');
            this.onChangeOptions();
            //this.trigger('updateShape');
        }
        

        var t = this;
        setTimeout(function() {
            t.attr(selector, { fill: "red", "font-size": 15 });
            setTimeout(function() {
                var fill = '#4b4a67';
                if (o.t == "in") fill = '#fff';
                t.attr(selector, { fill: fill, "font-size": 13 });
               // t.trigger('updateShape');
            }, 5000);
        }, 200);
    },

    onChangeOptions: function() {
        var outoptions = this.get('outOptions');
        var inoptions = this.get('inOptions');
        var size = this.get('size');
        var optionHeight = this.get('optionHeight');
        var optionRowHeight = this.get('optionRowHeight');

        var attrs = this.get('attrs');
        _.each(attrs, function(attrs, selector) {
            if (attrs.dynamic) {
                this.removeAttr(selector, { silent: true });
            }
        }, this);

        var offsetY = 0;
        var attrsUpdate = {};

        var t = this;
        _.each(inoptions, function(option) {
            var v = option.value;
            var text = "";

            if (_.isObject(v)) {
                text = getIcon(v.type)

                if (_.isString(v.value)) {
                    text += (text.length > 0 ? " " : "") + v.value;
                }
            }

            var selector = '.option-in-' + option.id;
            attrsUpdate[selector] = { transform: 'translate(0, ' + offsetY + ')', dynamic: true };
            attrsUpdate[selector + ' .option-rect'] = { height: optionHeight, dynamic: true };
            attrsUpdate[selector + ' .option-port .port-body'] = { port: "in-" + option.id, dynamic: true };
            attrsUpdate[selector + ' .option-text'] = { text: text, dynamic: true };
            var w = text.length * 7.4;

            var vv = "";
            if (_.isString(v.v)) vv = v.v;

            attrsUpdate[selector + ' .option-value'] = { text: vv, dynamic: true, dx: w + 10 };

            if (_.isObject(v) && v.type == "object" && v.value) {
                w = w + 30;
                attrsUpdate[selector + ' .icon-value'] = { addr: v.value };
            }
            option.width = w + 15 + vv.length * 7.4;

            offsetY += optionHeight;
        }, this);

        _.each(outoptions, function(option) {
            var selector = '.option-out-' + option.id;
            attrsUpdate[selector] = { transform: 'translate(0, ' + offsetY + ')', dynamic: true };

            option.width = 0;
            var h = 0;
            if (_.isArray(option.values)) {
                var offsetRowY = 0;
                _.each(option.values, function(value, i) {
                    var text = "";

                    if (_.isObject(value)) {
                        text = getIcon(value.type);
                        if (_.isString(value.value))
                            text += (text.length > 0 ? " " : "") + value.value;
                    }

                    offsetY += optionRowHeight;
                    offsetRowY += optionRowHeight;
                    h += optionRowHeight;
                    var selectorRow = selector + "-row-" + i;
                    //optionRow: '<g class="option-row row"><rect class="option-row-rect"/><text class="option-row-text"/><text class="option-row-value"/></g>',
                    attrsUpdate[selectorRow] = { transform: 'translate(0, ' + offsetRowY + ')', dynamic: true };
                    attrsUpdate[selectorRow + ' .option-row-text'] = { text: text, dynamic: true };

                    var w = text.length * 7.4;

                    var v = "";
                    if (_.isString(value.v)) v = value.v;

                    attrsUpdate[selectorRow + ' .option-row-value'] = { text: v, dynamic: true, dx: w + 10 };


                    if (value.type == "object") {
                        w = w + 30;
                        attrsUpdate[selectorRow + ' .icon-value'] = { addr: value.value };
                    }
                    option.width = Math.max(option.width, w + 15 + v.length * 7.4);

                }, this)
            }
            h += optionHeight;
            attrsUpdate[selector + ' .option-rect'] = { height: h, dynamic: true };
            attrsUpdate[selector + ' .option-port .port-body'] = { port: option.id, dynamic: true };
            attrsUpdate[selector + ' .option-text'] = { text: option.text, dynamic: true };
            if (option.portLabel) {
                attrsUpdate[selector + ' .option-port .port-label'] = { text: option.portLabel };
            }
            option.width = Math.max(option.width, 10 + option.text.length * 7.4);

            offsetY += optionHeight;
        }, this);

        //console.log(attrsUpdate);   
        this.attr(attrsUpdate);
        this.autoresize();
    },

    autoresize: function() {
        var options = this.get('inOptions') || [];
        var width = 0,
            w = 0;
        w = _.chain(options).map(function(o) {
                return o.width })
            .max().value();
        width = Math.max(w, width);

        var inoptions = this.get('outOptions') || [];
        w = 0;
        w = _.chain(inoptions).map(function(o) {
                return o.width })
            .max().value();
        width = Math.max(w, width);

        var rows = 0;
        _.each(inoptions, function(o) {
            if (_.isArray(o.values)) rows += o.values.length;
        })

        var gap = this.get('paddingBottom') || 20;
        var height = (options.length + inoptions.length) * this.get('optionHeight') + rows * this.get("optionRowHeight") + this.get('questionHeight') + gap;

        w = joint.util.measureText(this.get('question'), {
            fontSize: this.attr('.question-text/font-size')
        }).width;
        width = Math.max(this.get('minWidth'), w, width);
        width = Math.min(this.get('maxWidth'), width)
        this.attr({ ".icon-value": { "ref-x": width - 40 } })
        this.resize(width, height);

        this.trigger('updateShape');
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};

        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';

        var pl = this.get("labelEvent");

        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: pl } };

        if (selector === '.outPorts') { attrs[portSelector]['ref-dy'] = 0; }

        return attrs;
    },

    addOption: function(option) {
        var options = JSON.parse(JSON.stringify(this.get('options')));
        options.push(option);
        this.set('options', options);
    },

    removeOption: function(id) {
        var options = JSON.parse(JSON.stringify(this.get('options')));
        this.set('options', _.without(options, _.findWhere(options, { id: id })));
    },

    changeOption: function(id, option) {
        if (!option.id) {
            option.id = id;
        }
        var options = JSON.parse(JSON.stringify(this.get('options')));
        options[_.findIndex(options, { id: id })] = option;
        this.set('options', options);
    }
}));

joint.shapes.qad.QuestionView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    events: {
        'click .icon-value': function(e) {
            var t = $(e.currentTarget);
            addr = t.attr("addr");
            if (addr) {
                $.post("api/?a=get_addr", { addr: addr }, function(o) {
                    o = parseJSON(o);
                    if (_.isObject(o)) {
                        new app.ModalChangeValue({ datatype: o.datatype, address: o.address, value: o.value, name: o.name });
                    }
                })
            }
        }
    },

    //если изменился тип скрипта (event или timer)
    onChangeEventType: function() {
        var eventValue = this.model.get("eventValue");
        //если не таймер то порта вверху быть не должно 
        if (eventValue != "timer") {
            //this.$("g.eventport").remove();
            this.$(".inPorts").remove();
        } else {
            var p = this.elTopPortMarkup.clone();
            this.$(".inPorts").append(p.node);
            this.model.attr({ ".port-in-text": { text: getIcon("timer"), dx: -50, "font-size": 12, "font-family": "FontAwesome" } });
            this.update();
        }
    },

    initialize: function() {
        //app.FuncModel.updateShape 
        this.listenTo(this.model, "updateShape", this.onChangeEventType, this);
        this.listenTo(this.model, "change:eventValue", this.onChangeEventType, this);

        this.listenTo(this.model, 'change:inOptions', this.renderOptions, this);

        this.listenTo(this.model, 'change:outOptions', function() {
            this.renderOptions();

            var l = _.each(appView.graph.getLinks(), function(l) {
                var p = getTargetSource(l);
                if (p.src.port) {
                    var id = $(p.src.port).attr("id")
                    if (id) {
                        if ($("#" + id).length == 0) l.remove();
                    } else l.remove();
                } else l.remove();
            });

        });

        this.elTopPortMarkup = V(this.model.portMarkup);
        this.onChangeEventType();

        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    },

    renderMarkup: function() {
        joint.dia.ElementView.prototype.renderMarkup.apply(this, arguments);
        this.$options = this.$('.options');

        this.elInOption = V(this.model.inOptionMarkup);
        this.elInOptionWithoutPort = V(this.model.inOptionMarkupWithoutPort);

        this.elInOptionAddr = V(this.model.inOptionMarkupAddr);
        this.elInOptionAddrWithoutPort = V(this.model.inOptionMarkupAddrWithoutPort);

        this.elOutOption = V(this.model.outOptionMarkup);

        this.elRowOption = V(this.model.optionRow);
        this.elRowOptionAddr = V(this.model.optionRowAddr);

        this.renderOptions();
    },

    renderOptions: function() {
        this.$options.empty();
        var t = this;

        var inputs;
        if (this.model.get("m")) {
            inputs = this.model.get("m").get("input");
        } else {
            inputs = this.model.get("m_attr").input;
        }

        function addOption(option, isInput, i) {
            var className = 'option-' + (isInput ? "in" : "out") + "-" + option.id;

            var elOption;
            //для исходящих портов 
            if (!isInput) {
                elOption = t.elOutOption.clone().addClass(className);
                var rows = $(".option-rows", elOption.node);
                if (_.isArray(option.values)) {
                    _.each(option.values, function(v, i) {
                        var classRow = className + "-row-" + i;
                        var elRow;
                        if (v.type == "object") {
                            elRow = t.elRowOptionAddr.clone().addClass(classRow); //.data("addr", v.value);
                        } else
                            elRow = t.elRowOption.clone().addClass(classRow);
                        rows.append(elRow.node);
                    })
                }
            //входящие 
            }else {
                if (isInput && _.isArray(inputs) && inputs.length > i) {
                    var v = inputs[i].value;

                    if (_.isObject(option.value) && option.value.type == "object") {
                        if (v.value === false || v.link) { 
                            elOption = t.elInOptionAddr.clone().addClass(className);
                        } else { 
                            elOption = t.elInOptionAddrWithoutPort.clone().addClass(className);
                        }
                    }

                    if (_.isUndefined(elOption) && (v.value === false || v.link)) {
                        elOption = t.elInOption.clone().addClass(className);
                    }
                }

                if (_.isUndefined(elOption))
                    elOption = t.elInOptionWithoutPort.clone().addClass(className);
            }

            $(".port-body", $(elOption.node)).attr('option-id', option.id);
            elOption.attr('option-id', option.id);

            if (isInput && option.event) elOption.addClass('event');
            if (isInput && option.error) elOption.addClass('error');
            t.$options.append(elOption.node);
        }

        _.each(this.model.get('inOptions'), function(option, i) { addOption(option, true, i) }, this);
        _.each(this.model.get('outOptions'), function(option) { addOption(option, false) }, this);

        if (this.model.get("eventValue") != "timer")
            this.onChangeEventType();

        this.update();
       // this.model.trigger('updateShape');
    },
}));

joint.shapes.qad.Address = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/><g class="inPorts"/><text class="value"/><text class="icon-value"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/><text class="port-label"/></g>',

    defaults: joint.util.deepSupplement({
        value : false,
        type: 'qad.Address',
        size: { width: 155, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#ABDAC6', stroke: 'none', width: 155, height: 40 /*, rx: 3, ry: 3 */ },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61874), 'ref-x': 20, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.event.Address"), 'ref-x': 37, "ref-y": ".35"},

            '.outPorts': { "ref-y": .5, "ref-x": .99, ref:"rect"},
            '.outPorts .port-body': { fill: "orange", r: 14, type: 'out'},

            '.inPorts': { "ref-y": .5, "ref-x": .01, mag: 'passive' },
            '.inPorts .port-body': { fill: "#AAA", r: 14, type: 'in', mag: 'passive' },

            '.value': { text: "", "ref-y": ".35", "font-family": "FontAwesome", "font-size": 13, "font-style": "italic" },
            '.icon-value': { text: String.fromCharCode(61545), "ref-y": ".3", "font-family": "FontAwesome", cursor: "pointer", "font-size": 16 },
        },
        outPorts: [{ id: 'out', label: 'Yes' }],
        inPorts: [{ id: 'in', label: 'Yes' }],
        v : "",
        minWidth: 100,
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Address";
            this.attr('.label/text', a);
        }, this);

        //добавляем в маппинг
        var a = this.get("value");
        if (_.isString(a)) {
            if (!mapping.o[a]) mapping.o[a] = [];
            mapping.o[a].push({ model: this });
        }

        this.autoresize();
        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    //изменилось значение в групповам адресе
    changeValue: function(o, v) {
        var pv = this.get("v");
        this.set("v", v);
        this.attr('.value/text', v);

        this.autoresize();

        var t = this;

        if (pv.length > 0) {
            t.attr('.value', { fill: "red", "font-size": 15, "ref-y": .3 });
            _.each(appView.graph.getLinks(), function(l) {
                var p = getTargetSource(l);
                if (p.src && p.src.cell && p.src.cell == t) {
                    l.attr(".marker-target", { fill: "red", stroke: "red" });
                    l.attr(".connection", { stroke: "red" });
                }
            })

            setTimeout(function() {
                t.attr('.value', { fill: "black", "font-size": 13, "ref-y": .35 });
                _.each(appView.graph.getLinks(), function(l) {
                    var p = getTargetSource(l);
                    if (p.src && p.src.cell && p.src.cell == t) {
                        l.attr(".marker-target", { fill: "orange", stroke: "orange" });
                        l.attr(".connection", { stroke: "orange" });
                    }
                })

            }, 5000)
        }
    },

    autoresize: function() {
        var v = "";
        if (_.isString(this.get("value"))) v = this.get("value");
        var w = v.length * 10 + 25;
        this.attr('.value/dx', w);

        var v = "";
        if (_.isString(this.get("v"))) v = this.get("v");
        w += v.length * 8.4 + 12;


        this.attr('.icon-value/dx', w - 15);
        w += 20;

        w = Math.max(this.get("minWidth"), w);
        this.attr('rect/width', w);
        this.resize(w, 40);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};
        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';
        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    },
}));

joint.shapes.qad.AddressView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    },

    events: {
        "click text": function() {
            var addr = this.model.get("value");
            if (addr) {
                $.post("api/?a=get_addr", { addr: addr }, function(o) {
                    o = parseJSON(o);
                    if (_.isObject(o)) {
                        new app.ModalChangeValue({ datatype: o.datatype, address: o.address, value: o.value, name: o.name });
                    }
                })
            }
        }
    }
}));

joint.shapes.qad.Timer = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/></g>',

    defaults: joint.util.deepSupplement({
        value: 60,
        type: 'qad.Timer',
        size: { width: 115, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#DDCFE6', stroke: 'none', width: 115, height: 40 /*, rx: 3, ry: 3 */ },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61463), 'ref-x': 10, "ref-y": ".35" },
            '.label': { text: "60", 'ref-x': 27, "ref-y": ".35" },
            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': { fill: "orange", r: 14, type: 'out', isTimer: "1" },
        },
        outPorts: [{ id: 'out', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        joint.shapes.basic.Generic.prototype.initialize.apply(this, arguments);

        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = 0;
            this.attr('.label/text', a);
            var w = a.toString().length * 8.5 + 30;
            if (w < 115) w = 115;
            this.resize(w, 40);
            this.attr("rect/width", w);
        }, this);

        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};

        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';

        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };

        return attrs;
    },
}));

joint.shapes.qad.TimerView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));

joint.shapes.qad.Tag = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/></g>',

    defaults: joint.util.deepSupplement({
        value: false,
        type: 'qad.Tag',
        size: { width: 115, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#ABB7DD', stroke: 'none', width: 115, height: 40 },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61484), 'ref-x': 10, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.event_elements.Tag"), 'ref-x': 27, "ref-y": ".35" },
            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': { fill: 'orange', r: 14, type: 'out' },
        },
        outPorts: [{ id: 'out', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        joint.shapes.basic.Generic.prototype.initialize.apply(this, arguments);

        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Tag";
            this.attr('.label/text', a);
            var w = a.length * 8.5 + 30;
            if (w < 115) w = 115;
            this.resize(w, 40);
            this.attr("rect/width", w);
        }, this);

        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {

        var attrs = {};

        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';

        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    },
}));

joint.shapes.qad.TagView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));

joint.shapes.qad.Addressnotevent = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/><g class="inPorts"/><text class="value"/><text class="icon-value"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/><text class="port-label"/></g>',

    defaults: joint.util.deepSupplement({
        value: false,
        type: 'qad.Addressnotevent',
        size: { width: 115, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#ABDAC6', stroke: 'none', width: 155, height: 40 },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61874), 'ref-x': 20, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.event_elements.Address"), 'ref-x': 37, "ref-y": ".35" },

            '.outPorts': { "ref-y": .5, "ref-x": .99, ref:"rect" },
            '.outPorts .port-body': { fill: "#31d0c6", r: 14, type: 'out', notevent: false },

            '.inPorts': { "ref-y": .5, "ref-x": .01, mag: 'passive' },
            '.inPorts .port-body': { fill: "#AAA", r: 14, type: 'in', mag: 'passive' },

            '.value': { text: "", "ref-y": ".35", "font-family": "FontAwesome", "font-size": 13, "font-style": "italic" },
            '.icon-value': { text: String.fromCharCode(61545), "ref-y": ".3", "font-family": "FontAwesome", cursor: "pointer", "font-size": 16 },
        },
        outPorts: [{ id: 'out', label: 'Yes' }],
        inPorts: [{ id: 'in', label: 'Yes' }],
        v : "",
        minWidth: 100,
    }, joint.shapes.basic.Generic.prototype.defaults),

    initialize: function() {
        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Address";
            this.attr('.label/text', a);
        }, this);

        //добавляем в маппинг
        var a = this.get("value");
        if (_.isString(a)) {
            if (!mapping.o[a]) mapping.o[a] = [];
            mapping.o[a].push({ model: this });
        }

        this.autoresize();

        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    //изменилось значение в групповам адресе
    changeValue: function(o, v) {
        var pv = this.get("v");
        this.set("v", v);
        this.attr('.value/text', v);

        this.autoresize();

        var t = this;

        if (pv.length > 0) {
            t.attr('.value', { fill: "red", "font-size": 15, "ref-y": .3 });

            setTimeout(function() {
                t.attr('.value', { fill: "black", "font-size": 13, "ref-y": .35 });
            }, 5000)
        }
    },

    autoresize: function() {
        var v = "";
        if (_.isString(this.get("value"))) v = this.get("value");
        var w = v.length * 10 + 25;
        this.attr('.value/dx', w);

        var v = "";
        if (_.isString(this.get("v"))) v = this.get("v");
        w += v.length * 8.4 + 12;


        this.attr('.icon-value/dx', w - 15);
        w += 20;

        w = Math.max(this.get("minWidth"), w);
        this.attr('rect/width', w);
        this.resize(w, 40);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};
        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';
        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    }
}));

joint.shapes.qad.AddressnoteventView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    },

    events: {
        "click text": function() {
            var addr = this.model.get("value");
            if (addr) {
                $.post("api/?a=get_addr", { addr: addr }, function(o) {
                    o = parseJSON(o);
                    if (_.isObject(o)) {
                        new app.ModalChangeValue({ datatype: o.datatype, address: o.address, value: o.value, name: o.name });
                    }
                })
            }
        }
    }
}));

joint.shapes.qad.Storage = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/><g class="inPorts"/><text class="value"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/><text class="port-label"/></g>',

    defaults: joint.util.deepSupplement({
        value: false,
        type: 'qad.Storage',
        size: { width: 115, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#99CCCC', stroke: 'none', width: 115, height: 40 },
            'text': { fill: "black", },

            '.title': { text: String.fromCharCode(61888), 'ref-x': 20, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.event_elements.Storage"), 'ref-x': 37, "ref-y": ".35" },

            '.outPorts': { "ref-y": .5, "ref-x": .99, ref:"rect"  },
            '.outPorts .port-body': { fill: "#31d0c6", r: 14, type: 'out', notevent: false },

            '.inPorts': { "ref-y": .5, "ref-x": .01, mag: 'passive' },
            '.inPorts .port-body': { fill: "#AAA", r: 14, type: 'in', mag: 'passive' },

            '.value': { text: "", "ref-y": ".35", "font-family": "FontAwesome", "font-size": 13, "font-style": "italic" },
        },
        outPorts: [{ id: 'out', label: 'Yes' }],
        inPorts: [{ id: 'in', label: 'Yes' }],
        v : "",
        minWidth: 100,
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Storage";
            this.attr('.label/text', a);
        }, this);

        //добавляем в маппинг
        var a = this.get("value");
        if (_.isString(a)) {
            if (!mapping.s[a]) mapping.s[a] = [];
            mapping.s[a].push({ model: this });
        }

        this.autoresize();

        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    autoresize: function() {
        var v = "";
        if (_.isString(this.get("value"))) v = this.get("value");
        var w = v.length * 8 + 35;
        this.attr('.value/dx', w);

        var v = "";
        if (_.isString(this.get("v"))) v = this.get("v");
        w += v.length * 8.4 + 12;

        w = Math.max(this.get("minWidth"), w);
        this.attr('rect/width', w);
        this.resize(w, 40);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};
        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';
        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    },

    //изменилось значение в групповам адресе
    changeValue: function(o, v) {

        var pv = this.get("v");
        this.set("v", v);
        this.attr('.value/text', v);

        this.autoresize();

        var t = this;

        if (pv.length > 0) {
            t.attr('.value', { fill: "red", "font-size": 15, "ref-y": .3 });

            setTimeout(function() {
                t.attr('.value', { fill: "black", "font-size": 13, "ref-y": .35 });
            }, 5000)
        }
    },
}));

joint.shapes.qad.StorageView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));

joint.shapes.qad.Value = joint.shapes.basic.Generic.extend(_.extend({}, joint.shapes.basic.PortsModelInterface, {
    markup: '<g class="rotatable"><g><rect/></g><text class="title"/><text class="label"/><g class="outPorts"/></g>',
    portMarkup: '<g class="port port-<%= port.id %>"><circle class="port-body"/><text class="port-label"/></g>',

    defaults: joint.util.deepSupplement({
        value: false,
        type: 'qad.Value',
        size: { width: 115, height: 40 },
        attrs: {
            '.port-body': { magnet: true },
            'rect': { fill: '#CCFF66', stroke: 'none', width: 115, height: 40 },
            'text': { fill: "black", },
            '.title': { text: String.fromCharCode(61508), 'ref-x': 10, "ref-y": ".35" },
            '.label': { text: tr("leftmenu.elements.Value"), 'ref-x': 27, "ref-y": ".35" },
            '.outPorts': { "ref-y": .5, "ref-x": .99 },
            '.outPorts .port-body': { fill: "#31d0c6", r: 14, type: 'out', notevent: false },
        },
        outPorts: [{ id: 'out', label: 'Yes' }]
    }, joint.shapes.basic.Generic.prototype.defaults),


    initialize: function() {
        this.on('change:value', function() {
            var a = this.get("value");
            if (!a) a = "Value";
            this.attr('.label/text', a);
            var w = Math.max(114, a.length * 8 + 35);
            this.resize(w, 40);
            this.attr("rect/width", w);
        }, this);
        joint.shapes.basic.PortsModelInterface.initialize.apply(this, arguments);
    },

    getPortAttrs: function(port, index, total, selector, type) {
        var attrs = {};
        var portClass = 'port-' + port.id;
        var portSelector = selector + '>.' + portClass;
        var portTextSelector = portSelector + '>.port-label';
        var portCircleSelector = portSelector + '>.port-body';
        attrs[portCircleSelector] = { port: { id: port.id, type: type, label: port.label } };
        return attrs;
    }
}));

joint.shapes.qad.ValueView = joint.dia.ElementView.extend(_.extend({}, joint.shapes.basic.PortsViewInterface, {
    initialize: function() {
        joint.dia.ElementView.prototype.initialize.apply(this, arguments);
        joint.shapes.basic.PortsViewInterface.initialize.apply(this, arguments);
    }
}));
